@@ -4,6 +4,7 @@ import errno
 import os
 import platform
 import shutil
+import stat
 import subprocess
 import sys
 
@@ -24,13 +25,19 @@ def makedirs(path):
             return
         raise
 
+
 def rmtree(path):
+    def remove_readonly(func, path, excinfo):
+        os.chmod(path, stat.S_IWRITE)
+        func(path)
+
     try:
-        shutil.rmtree(path)
+        shutil.rmtree(path, onerror=remove_readonly)
     except OSError as e:
         if e.errno != errno.ENOENT:
             raise
 
+
 def exec_process(cmdline, silent=True, input=None, **kwargs):
     """Execute a subprocess and returns the returncode, stdout buffer and stderr buffer.
     Optionally prints stdout and stderr while running."""
@@ -107,34 +114,39 @@ class libuv_build_ext(build_ext):
     libuv_dir      = os.path.join('deps', 'libuv')
     libuv_repo     = 'https://github.com/libuv/libuv.git'
     libuv_branch   = 'v1.x'
-    libuv_revision = '0a8e813'
+    libuv_revision = '413dbd8'
     libuv_patches  = []
 
     user_options = build_ext.user_options
     user_options.extend([
         ("libuv-clean-compile", None, "Clean libuv tree before compilation"),
         ("libuv-force-fetch", None, "Remove libuv (if present) and fetch it again"),
-        ("libuv-verbose-build", None, "Print output of libuv build process")
+        ("libuv-verbose-build", None, "Print output of libuv build process"),
+        ("use-system-libuv", None, "Use the system provided libuv, instead of the bundled one")
     ])
     boolean_options = build_ext.boolean_options
-    boolean_options.extend(["libuv-clean-compile", "libuv-force-fetch", "libuv-verbose-build"])
+    boolean_options.extend(["libuv-clean-compile", "libuv-force-fetch", "libuv-verbose-build", "use-system-libuv"])
 
     def initialize_options(self):
         build_ext.initialize_options(self)
         self.libuv_clean_compile = 0
         self.libuv_force_fetch = 0
         self.libuv_verbose_build = 0
+        self.use_system_libuv = 0
 
     def build_extensions(self):
-        self.force = self.libuv_force_fetch or self.libuv_clean_compile
-        if sys.platform == 'win32':
-            self.libuv_lib = os.path.join(self.libuv_dir, 'Release', 'lib', 'libuv.lib')
+        self.force = self.force or self.libuv_force_fetch or self.libuv_clean_compile
+        if self.use_system_libuv:
+            self.compiler.add_library('uv')
         else:
-            self.libuv_lib = os.path.join(self.libuv_dir, '.libs', 'libuv.a')
-        self.get_libuv()
-        # Set compiler options
-        self.extensions[0].extra_objects.extend([self.libuv_lib])
-        self.compiler.add_include_dir(os.path.join(self.libuv_dir, 'include'))
+            if sys.platform == 'win32':
+                self.libuv_lib = os.path.join(self.libuv_dir, 'Release', 'lib', 'libuv.lib')
+            else:
+                self.libuv_lib = os.path.join(self.libuv_dir, '.libs', 'libuv.a')
+            self.get_libuv()
+            # Set compiler options
+            self.extensions[0].extra_objects.extend([self.libuv_lib])
+            self.compiler.add_include_dir(os.path.join(self.libuv_dir, 'include'))
         if sys.platform.startswith('linux'):
             self.compiler.add_library('rt')
         elif sys.platform == 'win32':
