@@ -1,26 +1,27 @@
 
 static void
-on_async_callback(uv_async_t *handle, int status)
+pyuv__async_cb(uv_async_t *handle)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     Async *self;
     PyObject *result;
 
     ASSERT(handle);
-    ASSERT(status == 0);
-
     self = PYUV_CONTAINER_OF(handle, Async, async_h);
 
-    /* Object could go out of scope in the callback, increase refcount to avoid it */
-    Py_INCREF(self);
+    if (self->callback != Py_None) {
+        /* Object could go out of scope in the callback, increase refcount to avoid it */
+        Py_INCREF(self);
 
-    result = PyObject_CallFunctionObjArgs(self->callback, self, NULL);
-    if (result == NULL) {
-        handle_uncaught_exception(HANDLE(self)->loop);
+        result = PyObject_CallFunctionObjArgs(self->callback, self, NULL);
+        if (result == NULL) {
+            handle_uncaught_exception(HANDLE(self)->loop);
+        }
+        Py_XDECREF(result);
+
+        Py_DECREF(self);
     }
-    Py_XDECREF(result);
 
-    Py_DECREF(self);
     PyGILState_Release(gstate);
 }
 
@@ -28,14 +29,14 @@ on_async_callback(uv_async_t *handle, int status)
 static PyObject *
 Async_func_send(Async *self)
 {
-    int r;
+    int err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    r = uv_async_send(&self->async_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_AsyncError);
+    err = uv_async_send(&self->async_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_AsyncError);
         return NULL;
     }
 
@@ -46,33 +47,32 @@ Async_func_send(Async *self)
 static int
 Async_tp_init(Async *self, PyObject *args, PyObject *kwargs)
 {
-    int r;
+    int err;
     Loop *loop;
-    PyObject *callback, *tmp;
+    PyObject *callback;
 
     UNUSED_ARG(kwargs);
-
     RAISE_IF_HANDLE_INITIALIZED(self, -1);
 
-    if (!PyArg_ParseTuple(args, "O!O:__init__", &LoopType, &loop, &callback)) {
+    callback = Py_None;
+
+    if (!PyArg_ParseTuple(args, "O!|O:__init__", &LoopType, &loop, &callback)) {
         return -1;
     }
 
-    if (!PyCallable_Check(callback)) {
+    if (callback != Py_None && !PyCallable_Check(callback)) {
         PyErr_SetString(PyExc_TypeError, "a callable is required");
         return -1;
     }
 
-    r = uv_async_init(loop->uv_loop, &self->async_h, on_async_callback);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_AsyncError);
+    err = uv_async_init(loop->uv_loop, &self->async_h, pyuv__async_cb);
+    if (err != 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_AsyncError);
         return -1;
     }
 
-    tmp = self->callback;
     Py_INCREF(callback);
     self->callback = callback;
-    Py_XDECREF(tmp);
 
     initialize_handle(HANDLE(self), loop);
 
