@@ -1,6 +1,6 @@
 
 static void
-on_pipe_connection(uv_stream_t* handle, int status)
+pyuv__pipe_listen_cb(uv_stream_t* handle, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     Pipe *self;
@@ -13,8 +13,7 @@ on_pipe_connection(uv_stream_t* handle, int status)
     Py_INCREF(self);
 
     if (status != 0) {
-        uv_err_t err = uv_last_error(UV_HANDLE_LOOP(self));
-        py_errorno = PyInt_FromLong((long)err.code);
+        py_errorno = PyInt_FromLong((long)status);
     } else {
         py_errorno = Py_None;
         Py_INCREF(Py_None);
@@ -33,7 +32,7 @@ on_pipe_connection(uv_stream_t* handle, int status)
 
 
 static void
-on_pipe_client_connection(uv_connect_t *req, int status)
+pyuv__pipe_connect_cb(uv_connect_t *req, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     Pipe *self;
@@ -46,8 +45,7 @@ on_pipe_client_connection(uv_connect_t *req, int status)
     ASSERT(self);
 
     if (status != 0) {
-        uv_err_t err = uv_last_error(UV_HANDLE_LOOP(self));
-        py_errorno = PyInt_FromLong(err.code);
+        py_errorno = PyInt_FromLong(status);
     } else {
         py_errorno = Py_None;
         Py_INCREF(Py_None);
@@ -70,63 +68,23 @@ on_pipe_client_connection(uv_connect_t *req, int status)
 }
 
 
-static void
-on_pipe_read2(uv_pipe_t* handle, int nread, uv_buf_t buf, uv_handle_type pending)
-{
-    PyGILState_STATE gstate = PyGILState_Ensure();
-    uv_err_t err;
-    Pipe *self;
-    PyObject *result, *data, *py_errorno, *py_pending;
-    ASSERT(handle);
-
-    self = PYUV_CONTAINER_OF(handle, Pipe, pipe_h);
-
-    /* Object could go out of scope in the callback, increase refcount to avoid it */
-    Py_INCREF(self);
-
-    py_pending = PyInt_FromLong((long)pending);
-
-    if (nread >= 0) {
-        data = PyBytes_FromStringAndSize(buf.base, nread);
-        py_errorno = Py_None;
-        Py_INCREF(Py_None);
-    } else {
-        data = Py_None;
-        Py_INCREF(Py_None);
-        err = uv_last_error(UV_HANDLE_LOOP(self));
-        py_errorno = PyInt_FromLong((long)err.code);
-    }
-
-    result = PyObject_CallFunctionObjArgs(((Stream *)self)->on_read_cb, self, data, py_pending, py_errorno, NULL);
-    if (result == NULL) {
-        handle_uncaught_exception(HANDLE(self)->loop);
-    }
-    Py_XDECREF(result);
-    Py_DECREF(data);
-    Py_DECREF(py_pending);
-    Py_DECREF(py_errorno);
-
-    Py_DECREF(self);
-    PyGILState_Release(gstate);
-}
-
-
 static PyObject *
 Pipe_func_bind(Pipe *self, PyObject *args)
 {
-    int r;
+    int err;
     char *name;
+    Py_ssize_t len;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    if (!PyArg_ParseTuple(args, "s:bind", &name)) {
+    if (!PyArg_ParseTuple(args, "s#:bind", &name, &len)) {
         return NULL;
     }
 
-    r = uv_pipe_bind(&self->pipe_h, name);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_PipeError);
+    err = uv_pipe_bind(&self->pipe_h, name);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
         return NULL;
     }
 
@@ -137,10 +95,10 @@ Pipe_func_bind(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_listen(Pipe *self, PyObject *args)
 {
-    int r, backlog;
+    int err, backlog;
     PyObject *callback, *tmp;
 
-    backlog = 128;
+    backlog = 511;
     tmp = NULL;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -160,9 +118,9 @@ Pipe_func_listen(Pipe *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_listen((uv_stream_t *)&self->pipe_h, backlog, on_pipe_connection);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_PipeError);
+    err = uv_listen((uv_stream_t *)&self->pipe_h, backlog, pyuv__pipe_listen_cb);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
         return NULL;
     }
 
@@ -178,7 +136,7 @@ Pipe_func_listen(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_accept(Pipe *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *client;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -200,9 +158,9 @@ Pipe_func_accept(Pipe *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_accept((uv_stream_t *)&self->pipe_h, (uv_stream_t *)UV_HANDLE(client));
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_PipeError);
+    err = uv_accept((uv_stream_t *)&self->pipe_h, (uv_stream_t *)UV_HANDLE(client));
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
         return NULL;
     }
 
@@ -240,7 +198,7 @@ Pipe_func_connect(Pipe *self, PyObject *args)
 
     connect_req->data = callback;
 
-    uv_pipe_connect(connect_req, &self->pipe_h, name, on_pipe_client_connection);
+    uv_pipe_connect(connect_req, &self->pipe_h, name, pyuv__pipe_connect_cb);
 
     /* Increase refcount so that object is not removed before the callback is called */
     Py_INCREF(self);
@@ -252,7 +210,7 @@ Pipe_func_connect(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_open(Pipe *self, PyObject *args)
 {
-    int r;
+    int err;
     long fd;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -262,9 +220,9 @@ Pipe_func_open(Pipe *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_pipe_open(&self->pipe_h, (uv_file)fd);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_PipeError);
+    err = uv_pipe_open(&self->pipe_h, (uv_file)fd);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
         return NULL;
     }
 
@@ -292,98 +250,196 @@ Pipe_func_pending_instances(Pipe *self, PyObject *args)
 
 
 static PyObject *
-Pipe_func_start_read2(Pipe *self, PyObject *args)
+Pipe_func_pending_handle_type(Pipe *self)
 {
-    int r;
-    PyObject *tmp, *callback;
-
-    tmp = NULL;
-
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    if (!PyArg_ParseTuple(args, "O:start_read2", &callback)) {
-        return NULL;
-    }
-
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
-    }
-
-    r = uv_read2_start((uv_stream_t *)&self->pipe_h, (uv_alloc_cb)on_stream_alloc, (uv_read2_cb)on_pipe_read2);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_PipeError);
-        return NULL;
-    }
-
-    tmp = ((Stream *)self)->on_read_cb;
-    Py_INCREF(callback);
-    ((Stream *)self)->on_read_cb = callback;
-    Py_XDECREF(tmp);
-
-    Py_RETURN_NONE;
+    return PyInt_FromLong(uv_pipe_pending_type(&self->pipe_h));
 }
 
 
 static PyObject *
-Pipe_func_write2(Pipe *self, PyObject *args)
+Pipe_func_write(Pipe *self, PyObject *args)
 {
-    uv_buf_t buf;
-    stream_write_ctx *ctx;
-    Py_buffer *view;
+    PyObject *data;
     PyObject *callback, *send_handle;
 
-    callback = Py_None;
-
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    ctx = PyMem_Malloc(sizeof *ctx);
-    if (!ctx) {
-        PyErr_NoMemory();
+    callback = send_handle = Py_None;
+
+    if (!PyArg_ParseTuple(args, "O|OO:write", &data, &callback, &send_handle)) {
         return NULL;
     }
 
-    view = &ctx->view[0];
-
-    if (!PyArg_ParseTuple(args, PYUV_BYTES"*O|O:write", view, &send_handle, &callback)) {
-        PyMem_Free(ctx);
+    if (callback != Py_None && !PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "'callback' must be a callable or None");
         return NULL;
     }
 
-    if (PyObject_IsSubclass((PyObject *)send_handle->ob_type, (PyObject *)&StreamType)) {
+    if (send_handle == Py_None) {
+        send_handle = NULL;
+    } else if (PyObject_IsSubclass((PyObject *)send_handle->ob_type, (PyObject *)&StreamType)) {
         if (UV_HANDLE(send_handle)->type != UV_TCP && UV_HANDLE(send_handle)->type != UV_NAMED_PIPE) {
-            PyErr_SetString(PyExc_TypeError, "Only TCP and Pipe objects are supported for write2");
-            goto error;
+            PyErr_SetString(PyExc_TypeError, "Only TCP and Pipe objects are supported");
+            return NULL;
         }
     } else if (PyObject_IsSubclass((PyObject *)send_handle->ob_type, (PyObject *)&UDPType)) {
         /* empty */
     } else {
         PyErr_SetString(PyExc_TypeError, "Only Stream and UDP objects are supported");
-        goto error;
-    }
-
-    if (callback != Py_None && !PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable or None is required");
-        goto error;
-    }
-
-    buf = uv_buf_init(view->buf, view->len);
-
-    return pyuv_stream_write((Stream *)self, ctx, view, &buf, 1, callback, send_handle);
-
-error:
-        PyBuffer_Release(view);
-        PyMem_Free(ctx);
         return NULL;
+    }
+
+    if (PyObject_CheckBuffer(data)) {
+        return pyuv__stream_write_bytes((Stream *)self, data, callback, send_handle);
+    } else if (!PyUnicode_Check(data) && PySequence_Check(data)) {
+        return pyuv__stream_write_sequence((Stream *)self, data, callback, send_handle);
+    } else {
+        PyErr_SetString(PyExc_TypeError, "only bytes and sequences are supported");
+        return NULL;
+    }
+}
+
+
+static PyObject *
+Pipe_func_getsockname(Pipe *self)
+{
+#ifdef _WIN32
+    /* MAX_PATH is in characters, not bytes. Make sure we have enough headroom. */
+    char buf[MAX_PATH * 4];
+#else
+    char buf[PATH_MAX];
+#endif
+    size_t buf_len;
+    int err;
+
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
+    RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
+
+    buf_len = sizeof(buf);
+    err = uv_pipe_getsockname(&self->pipe_h, buf, &buf_len);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
+        return NULL;
+    }
+
+    if (buf_len == 0) {
+        return PyBytes_FromString("");
+    } else if (buf[0] == '\0') {
+        /* Linux abstract namespace */
+        return PyBytes_FromStringAndSize(buf, buf_len);
+    } else {
+#ifdef PYUV_PYTHON3
+        return PyUnicode_DecodeFSDefaultAndSize(buf, buf_len-1);
+#else
+        return PyBytes_FromStringAndSize(buf, buf_len-1);
+#endif
+    }
+
+}
+
+
+static PyObject *
+Pipe_sndbuf_get(Pipe *self, void *closure)
+{
+    int err;
+    int sndbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
+
+    sndbuf_value = 0;
+    err = uv_send_buffer_size(UV_HANDLE(self), &sndbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
+        return NULL;
+    }
+    return PyInt_FromLong((long) sndbuf_value);
+}
+
+
+static int
+Pipe_sndbuf_set(Pipe *self, PyObject *value, void *closure)
+{
+    int err;
+    int sndbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, -1);
+
+    if (!value) {
+        PyErr_SetString(PyExc_TypeError, "cannot delete attribute");
+        return -1;
+    }
+
+    sndbuf_value = (int) PyInt_AsLong(value);
+    if (sndbuf_value == -1 && PyErr_Occurred()) {
+        return -1;
+    }
+
+    err = uv_send_buffer_size(UV_HANDLE(self), &sndbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
+        return -1;
+    }
+    return 0;
+}
+
+
+static PyObject *
+Pipe_rcvbuf_get(Pipe *self, void *closure)
+{
+    int err;
+    int rcvbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
+
+    rcvbuf_value = 0;
+    err = uv_recv_buffer_size(UV_HANDLE(self), &rcvbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
+        return NULL;
+    }
+    return PyInt_FromLong((long) rcvbuf_value);
+}
+
+
+static int
+Pipe_rcvbuf_set(Pipe *self, PyObject *value, void *closure)
+{
+    int err;
+    int rcvbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, -1);
+
+    if (!value) {
+        PyErr_SetString(PyExc_TypeError, "cannot delete attribute");
+        return -1;
+    }
+
+    rcvbuf_value = (int) PyInt_AsLong(value);
+    if (rcvbuf_value == -1 && PyErr_Occurred()) {
+        return -1;
+    }
+
+    err = uv_recv_buffer_size(UV_HANDLE(self), &rcvbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
+        return -1;
+    }
+    return 0;
 }
 
 
 static int
 Pipe_tp_init(Pipe *self, PyObject *args, PyObject *kwargs)
 {
-    int r;
+    int err;
     Loop *loop;
     PyObject *ipc = Py_False;
 
@@ -395,9 +451,9 @@ Pipe_tp_init(Pipe *self, PyObject *args, PyObject *kwargs)
         return -1;
     }
 
-    r = uv_pipe_init(loop->uv_loop, &self->pipe_h, (ipc == Py_True) ? 1 : 0);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_PipeError);
+    err = uv_pipe_init(loop->uv_loop, &self->pipe_h, (ipc == Py_True) ? 1 : 0);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_PipeError);
         return -1;
     }
 
@@ -448,12 +504,20 @@ Pipe_tp_methods[] = {
     { "connect", (PyCFunction)Pipe_func_connect, METH_VARARGS, "Start connecion to the remote Pipe." },
     { "open", (PyCFunction)Pipe_func_open, METH_VARARGS, "Open the specified file descriptor and manage it as a Pipe." },
     { "pending_instances", (PyCFunction)Pipe_func_pending_instances, METH_VARARGS, "Set the number of pending pipe instance handles when the pipe server is waiting for connections." },
-    { "start_read2", (PyCFunction)Pipe_func_start_read2, METH_VARARGS, "Extended read methods for receiving handles over a pipe. The pipe must be initialized with ipc set to True." },
-    { "write2", (PyCFunction)Pipe_func_write2, METH_VARARGS, "Write data and send handle over a pipe." },
+    { "pending_handle_type", (PyCFunction)Pipe_func_pending_handle_type, METH_NOARGS, "Returns the type of the next pending handle. Can be called multiple times." },
+    { "write", (PyCFunction)Pipe_func_write, METH_VARARGS, "Write data and send handle over a pipe." },
+    { "getsockname", (PyCFunction)Pipe_func_getsockname, METH_NOARGS, "Get bound pipe name." },
     { NULL }
 };
 
 
+static PyGetSetDef Pipe_tp_getsets[] = {
+    {"send_buffer_size", (getter)Pipe_sndbuf_get, (setter)Pipe_sndbuf_set, "Send buffer size.", NULL},
+    {"receive_buffer_size", (getter)Pipe_rcvbuf_get, (setter)Pipe_rcvbuf_set, "Receive buffer size.", NULL},
+    {NULL}
+};
+
+
 static PyTypeObject PipeType = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "pyuv.Pipe",                                                   /*tp_name*/
@@ -484,7 +548,7 @@ static PyTypeObject PipeType = {
     0,                                                             /*tp_iternext*/
     Pipe_tp_methods,                                               /*tp_methods*/
     0,                                                             /*tp_members*/
-    0,                                                             /*tp_getsets*/
+    Pipe_tp_getsets,                                               /*tp_getsets*/
     0,                                                             /*tp_base*/
     0,                                                             /*tp_dict*/
     0,                                                             /*tp_descr_get*/
