@@ -60,6 +60,7 @@ StdIO_stream_get(StdIO *self, void *closure)
     if (!self->stream) {
         Py_RETURN_NONE;
     } else {
+        Py_INCREF(self->stream);
         return self->stream;
     }
 }
@@ -168,24 +169,12 @@ static PyTypeObject StdIOType = {
 
 /* Process handle */
 
-#define RAISE_IF_SPAWNED(obj, retval)                                               \
-    do {                                                                            \
-        if ((obj)->spawned) {                                                       \
-            PyErr_SetString(PyExc_ProcessError, "Process already spawned");         \
-            return retval;                                                          \
-        }                                                                           \
-    } while(0)                                                                      \
+/* forward declaration */
+static PyObject* Process_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs);
 
-#define RAISE_IF_NOT_SPAWNED(obj, retval)                                           \
-    do {                                                                            \
-        if (!((obj)->spawned)) {                                                    \
-            PyErr_SetString(PyExc_ProcessError, "Process was not spawned");         \
-            return retval;                                                          \
-        }                                                                           \
-    } while(0)                                                                      \
 
 static void
-on_process_exit(uv_process_t *handle, int exit_status, int term_signal)
+pyuv__process_exit_cb(uv_process_t *handle, int64_t exit_status, int term_signal)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     Process *self;
@@ -216,28 +205,25 @@ on_process_exit(uv_process_t *handle, int exit_status, int term_signal)
 
 
 static PyObject *
-Process_func_spawn(Process *self, PyObject *args, PyObject *kwargs)
+Process_func_spawn(PyObject *cls, PyObject *args, PyObject *kwargs)
 {
-    int r, flags, len, stdio_count;
+    int err, flags, stdio_count;
     unsigned int uid, gid;
-    char *cwd, *cwd2, *file, *file2, *arg_str, *tmp_str, *key_str, *value_str;
-    char **ptr, **process_args, **process_env;
-    Py_ssize_t i, n, pos;
-    PyObject *key, *value, *item, *tmp, *callback, *arguments, *env, *stdio, *ret;
+    Py_ssize_t i, n, pos, size;
+    PyObject *key, *value, *item, *tmp, *callback, *arguments, *env, *stdio, *ret, *executable, *cwd;
+    Process *self;
+    Loop *loop;
     uv_process_options_t options;
     uv_stdio_container_t *stdio_container;
 
-    static char *kwlist[] = {"file", "exit_callback", "args", "env", "cwd", "uid", "gid", "flags", "stdio", NULL};
+    static char *kwlist[] = {"loop", "args", "executable", "env", "cwd", "uid", "gid", "flags", "stdio", "exit_callback", NULL};
 
-    cwd = NULL;
-    ptr = process_args = process_env = NULL;
+    cwd = executable = Py_None;
     tmp = arguments = env = stdio = NULL;
     stdio_container = NULL;
     flags = uid = gid = stdio_count = 0;
 
-    RAISE_IF_SPAWNED(self, NULL);
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|OOO!sIIiO:__init__", kwlist, &file, &callback, &arguments, &PyDict_Type, &env, &cwd, &uid, &gid, &flags, &stdio)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!O|OO!OIIiOO:__init__", kwlist, &LoopType, &loop, &arguments, &executable, &PyDict_Type, &env, &cwd, &uid, &gid, &flags, &stdio, &callback)) {
         return NULL;
     }
 
@@ -246,8 +232,8 @@ Process_func_spawn(Process *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-    if (arguments && !PySequence_Check(arguments)) {
-        PyErr_SetString(PyExc_TypeError, "only iterable objects are supported for 'args'");
+    if (!PyBytes_Check(arguments) && !PyUnicode_Check(arguments) && !PySequence_Check(arguments)) {
+        PyErr_SetString(PyExc_TypeError, "only string or iterable objects are supported for 'args'");
         return NULL;
     }
 
@@ -256,115 +242,145 @@ Process_func_spawn(Process *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
+    self = (Process *)Process_tp_new((PyTypeObject *) cls, args, kwargs);
+    if (!self) {
+        return NULL;
+    }
+
+    initialize_handle(HANDLE(self), loop);
+    /* Don't consider the handle initialized until uv_spawn is called. Unline other handles,
+     * there is no uv_process_init, it's called at the begining of uv_spawn, so don't
+     * consider the handle initialized until then, or we'd call uv_close when the Python
+     * object is deallocated, which is not A Good Thing (TM).
+     */
+    HANDLE(self)->initialized = False;
+
     memset(&options, 0, sizeof(uv_process_options_t));
 
     options.uid = uid;
     options.gid = gid;
     options.flags = flags;
-    options.exit_cb = on_process_exit;
+    options.exit_cb = pyuv__process_exit_cb;
 
-    file2 = PyMem_Malloc(strlen(file) + 1);
-    if (!file2) {
-        PyErr_NoMemory();
-        ret = NULL;
-        goto cleanup;
-    }
-    strcpy(file2, file);
-    options.file = file2;
+    /* process args */
 
-    if (arguments) {
-        n = PySequence_Length(arguments);
-        process_args = PyMem_Malloc(sizeof *process_args * (n + 2));
-        if (!process_args) {
+    if (PyBytes_Check(arguments) || PyUnicode_Check(arguments)) {
+        options.args = PyMem_Malloc(sizeof *(options.args) * 2);
+        if (!options.args) {
             PyErr_NoMemory();
-            PyMem_Free(file2);
-            ret = NULL;
-            goto cleanup;
+            goto error;
         }
-        process_args[0] = file2;
-        i = 0;
-        while (i < n) {
-            item = PySequence_GetItem(arguments, i);
-            if (!item || !PyArg_Parse(item, "s;args contains a non-string value", &arg_str)) {
-                Py_XDECREF(item);
-                ret = NULL;
-                goto cleanup;
-            }
-            tmp_str = PyMem_Malloc(strlen(arg_str) + 1);
-            if (!tmp_str) {
-                Py_DECREF(item);
-                ret = NULL;
-                goto cleanup;
-            }
-            strcpy(tmp_str, arg_str);
-            process_args[i+1] = tmp_str;
-            Py_DECREF(item);
-            i++;
+        options.args[0] = pyuv_dup_strobj(arguments);
+        if (!options.args[0]) {
+            goto error;
         }
-        process_args[i+1] = NULL;
+        options.args[1] = NULL;
     } else {
-        process_args = PyMem_Malloc(sizeof *process_args * 2);
-        if (!process_args) {
-            PyErr_NoMemory();
-            PyMem_Free(file2);
-            ret = NULL;
-            goto cleanup;
+        /* it's a sequence object */
+        n = PySequence_Length(arguments);
+        if (n < 1) {
+            PyErr_SetString(PyExc_ValueError, "'args' must contain at least one element");
+            goto error;
         }
-        process_args[0] = file2;
-        process_args[1] = NULL;
+        options.args = PyMem_Malloc(sizeof *(options.args) * (n + 1));
+        if (!options.args) {
+            PyErr_NoMemory();
+            goto error;
+        }
+        for (i = 0; i < n; i++) {
+            item = PySequence_GetItem(arguments, i);
+            if (!item) {
+                options.args[i] = NULL;
+                goto error;
+            }
+            options.args[i] = pyuv_dup_strobj(item);
+            if (!options.args[i]) {
+                Py_DECREF(item);
+                goto error;
+            }
+            Py_DECREF(item);
+        }
+        options.args[n] = NULL;
     }
-    options.args = process_args;
 
-    if (cwd) {
-        cwd2 = PyMem_Malloc(strlen(cwd) + 1);
-        if (!cwd2) {
-            PyErr_NoMemory();
-            ret = NULL;
-            goto cleanup;
+    /* process file */
+
+    if (executable != Py_None) {
+        options.file = pyuv_dup_strobj(executable);
+        if (!options.file) {
+            goto error;
         }
-        strcpy(cwd2, cwd);
-        options.cwd = cwd2;
+    } else {
+        size = strlen(options.args[0]) + 1;
+        options.file = PyMem_Malloc(size);
+        if (!options.file) {
+            PyErr_NoMemory();
+            goto error;
+        }
+        memcpy((void*)options.file, options.args[0], size);
     }
 
+    /* process cwd */
+    if (cwd != Py_None) {
+        options.cwd = pyuv_dup_strobj(cwd);
+        if (!options.cwd) {
+            goto error;
+        }
+    }
+
+    /* process env */
+
     if (env) {
+        char *key_str, *value_str;
+        PyObject *key_bytes, *value_bytes;
         n = PyDict_Size(env);
         if (n > 0) {
-            process_env = PyMem_Malloc(sizeof *process_env * (n + 1));
-            if (!process_env) {
+            options.env = PyMem_Malloc(sizeof *(options.env) * (n + 1));
+            if (!options.env) {
                 PyErr_NoMemory();
-                ret = NULL;
-                goto cleanup;
+                goto error;
             }
             i = 0;
             pos = 0;
             while (PyDict_Next(env, &pos, &key, &value)) {
-                if (!PyArg_Parse(key, "s;env contains a non-string key", &key_str) || !PyArg_Parse(value, "s;env contains a non-string value", &value_str)) {
-                    ret = NULL;
-                    goto cleanup;
+                key_bytes = value_bytes = NULL;
+                if (!pyuv_PyUnicode_FSConverter(key, &key_bytes)) {
+                    options.env[i] = NULL;
+                    goto error;
                 }
-                len = strlen(key_str) + strlen(value_str) + 2;
-                tmp_str = PyMem_Malloc(len);
-                if (!tmp_str) {
+                if (!pyuv_PyUnicode_FSConverter(value, &value_bytes)) {
+                    Py_DECREF(key_bytes);
+                    options.env[i] = NULL;
+                    goto error;
+                }
+                key_str = PyBytes_AS_STRING(key_bytes);
+                value_str = PyBytes_AS_STRING(value_bytes);
+                size = PyBytes_GET_SIZE(key_bytes) + PyBytes_GET_SIZE(value_bytes) + 2;
+                options.env[i] = PyMem_Malloc(size);
+                if (!options.env[i]) {
+                    options.env[i] = NULL;
                     PyErr_NoMemory();
-                    ret = NULL;
-                    goto cleanup;
+                    Py_DECREF(key_bytes);
+                    Py_DECREF(value_bytes);
+                    goto error;
                 }
-                PyOS_snprintf(tmp_str, len, "%s=%s", key_str, value_str);
-                process_env[i] = tmp_str;
+                PyOS_snprintf(options.env[i], size, "%s=%s", key_str, value_str);
+                Py_DECREF(key_bytes);
+                Py_DECREF(value_bytes);
                 i++;
             }
-            process_env[i] = NULL;
+            options.env[i] = NULL;
         }
     }
-    options.env = process_env;
+
+    /* process stdio container */
 
     if (stdio) {
         n = PySequence_Length(stdio);
         stdio_container = PyMem_Malloc(sizeof *stdio_container * n);
         if (!stdio_container) {
             PyErr_NoMemory();
-            ret = NULL;
-            goto cleanup;
+            goto error;
         }
         item = NULL;
         for (i = 0;i < n; i++) {
@@ -372,8 +388,7 @@ Process_func_spawn(Process *self, PyObject *args, PyObject *kwargs)
             if (!item || !PyObject_TypeCheck(item, &StdIOType)) {
                 Py_XDECREF(item);
                 PyErr_SetString(PyExc_TypeError, "a StdIO instance is required");
-                ret = NULL;
-                goto cleanup;
+                goto error;
             }
             stdio_count++;
             stdio_container[i].flags = ((StdIO *)item)->flags;
@@ -388,13 +403,16 @@ Process_func_spawn(Process *self, PyObject *args, PyObject *kwargs)
     options.stdio = stdio_container;
     options.stdio_count = stdio_count;
 
-    r = uv_spawn(UV_HANDLE_LOOP(self), &self->process_h, options);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_ProcessError);
-        ret = NULL;
-        goto cleanup;
+    HANDLE(self)->initialized = True;
+
+    err = uv_spawn(UV_HANDLE_LOOP(self), &self->process_h, &options);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_ProcessError);
+        goto error;
     }
 
+    ret = (PyObject *) self;
+
     tmp = (PyObject *)self->on_exit_cb;
     Py_INCREF(callback);
     self->on_exit_cb = callback;
@@ -405,33 +423,34 @@ Process_func_spawn(Process *self, PyObject *args, PyObject *kwargs)
     self->stdio = stdio;
     Py_XDECREF(tmp);
 
-    HANDLE(self)->initialized = True;
-    self->spawned = True;
-
-    ret = Py_None;
-
     /* Increase refcount so that object is not removed before the exit callback is called */
     Py_INCREF(self);
 
+    goto cleanup;
+
+error:
+    ret = NULL;
+    Py_DECREF(self);
+
 cleanup:
     if (options.args) {
-        for (ptr = options.args; *ptr != NULL; ptr++) {
-            PyMem_Free(*ptr);
+        for (i = 0; options.args[i] != NULL; ++i) {
+            PyMem_Free(options.args[i]);
         }
+        PyMem_Free(options.args);
     }
 
     if (options.env) {
-        for (ptr = options.env; *ptr != NULL; ptr++) {
-            PyMem_Free(*ptr);
+        for (i = 0; options.env[i] != NULL; ++i) {
+            PyMem_Free(options.env[i]);
         }
+        PyMem_Free(options.env);
     }
 
-    PyMem_Free(options.args);
-    PyMem_Free(options.cwd);
-    PyMem_Free(options.env);
+    PyMem_Free((void*)options.cwd);
+    PyMem_Free((void*)options.file);
     PyMem_Free(options.stdio);
 
-    Py_XINCREF(ret);
     return ret;
 }
 
@@ -439,19 +458,18 @@ cleanup:
 static PyObject *
 Process_func_kill(Process *self, PyObject *args)
 {
-    int signum, r;
+    int signum, err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
-    RAISE_IF_NOT_SPAWNED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
     if (!PyArg_ParseTuple(args, "i:kill", &signum)) {
         return NULL;
     }
 
-    r = uv_process_kill(&self->process_h, signum);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_ProcessError);
+    err = uv_process_kill(&self->process_h, signum);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_ProcessError);
         return NULL;
     }
 
@@ -459,40 +477,30 @@ Process_func_kill(Process *self, PyObject *args)
 }
 
 
+static PyObject *
+Process_func_close(Process *self, PyObject *args)
+{
+    if (!HANDLE(self)->initialized) {
+        PyErr_SetString(PyExc_RuntimeError, "Object was not initialized, spawn was not called.");
+        return NULL;
+    }
+
+    return Handle_func_close(HANDLE(self), args);
+}
+
+
 static PyObject *
 Process_pid_get(Process *self, void *closure)
 {
     UNUSED_ARG(closure);
 
-    if (!HANDLE(self)->initialized || !self->spawned) {
+    if (!HANDLE(self)->initialized) {
         Py_RETURN_NONE;
     }
     return PyInt_FromLong((long)self->process_h.pid);
 }
 
 
-static int
-Process_tp_init(Process *self, PyObject *args, PyObject *kwargs)
-{
-    Loop *loop;
-
-    UNUSED_ARG(kwargs);
-
-    RAISE_IF_HANDLE_INITIALIZED(self, -1);
-
-    if (!PyArg_ParseTuple(args, "O!:__init__", &LoopType, &loop)) {
-        return -1;
-    }
-
-    initialize_handle(HANDLE(self), loop);
-
-    /* uv_process_t handles are not initialized explicitly, so workaround it. See tp_dealloc for the rest */
-    HANDLE(self)->initialized = False;
-
-    return 0;
-}
-
-
 static PyObject *
 Process_func_disable_stdio_inheritance(PyObject *cls)
 {
@@ -514,20 +522,11 @@ Process_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 
     self->process_h.data = self;
     UV_HANDLE(self) = (uv_handle_t *)&self->process_h;
-    self->spawned = False;
 
     return (PyObject *)self;
 }
 
 
-static void
-Process_tp_dealloc(Process *self)
-{
-    HANDLE(self)->initialized = self->spawned;
-    HandleType.tp_dealloc((PyObject *)self);
-}
-
-
 static int
 Process_tp_traverse(Process *self, visitproc visit, void *arg)
 {
@@ -548,8 +547,9 @@ Process_tp_clear(Process *self)
 
 static PyMethodDef
 Process_tp_methods[] = {
-    { "spawn", (PyCFunction)Process_func_spawn, METH_VARARGS|METH_KEYWORDS, "Spawn the child process." },
+    { "spawn", (PyCFunction)Process_func_spawn, METH_CLASS|METH_VARARGS|METH_KEYWORDS, "Spawn the child process." },
     { "kill", (PyCFunction)Process_func_kill, METH_VARARGS, "Kill this process with the specified signal number." },
+    { "close", (PyCFunction)Process_func_close, METH_VARARGS, "Close process handle." },
     { "disable_stdio_inheritance", (PyCFunction)Process_func_disable_stdio_inheritance, METH_NOARGS|METH_CLASS, "Disables inheritance for file descriptors / handles that this process inherited from its parent." },
     { NULL }
 };
@@ -566,7 +566,7 @@ static PyTypeObject ProcessType = {
     "pyuv.Process",                                                 /*tp_name*/
     sizeof(Process),                                                /*tp_basicsize*/
     0,                                                              /*tp_itemsize*/
-    (destructor)Process_tp_dealloc,                                 /*tp_dealloc*/
+    0,                                                              /*tp_dealloc*/
     0,                                                              /*tp_print*/
     0,                                                              /*tp_getattr*/
     0,                                                              /*tp_setattr*/
@@ -597,7 +597,7 @@ static PyTypeObject ProcessType = {
     0,                                                              /*tp_descr_get*/
     0,                                                              /*tp_descr_set*/
     0,                                                              /*tp_dictoffset*/
-    (initproc)Process_tp_init,                                      /*tp_init*/
+    0,                                                              /*tp_init*/
     0,                                                              /*tp_alloc*/
     Process_tp_new,                                                 /*tp_new*/
 };
