@@ -1,6 +1,6 @@
 
 static void
-on_tcp_connection(uv_stream_t *handle, int status)
+pyuv__tcp_listen_cb(uv_stream_t *handle, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     TCP *self;
@@ -14,8 +14,7 @@ on_tcp_connection(uv_stream_t *handle, int status)
     Py_INCREF(self);
 
     if (status != 0) {
-        uv_err_t err = uv_last_error(UV_HANDLE_LOOP(self));
-        py_errorno = PyInt_FromLong((long)err.code);
+        py_errorno = PyInt_FromLong((long)status);
     } else {
         py_errorno = Py_None;
         Py_INCREF(Py_None);
@@ -34,7 +33,7 @@ on_tcp_connection(uv_stream_t *handle, int status)
 
 
 static void
-on_tcp_client_connection(uv_connect_t *req, int status)
+pyuv__tcp_connect_cb(uv_connect_t *req, int status)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     TCP *self;
@@ -45,8 +44,7 @@ on_tcp_client_connection(uv_connect_t *req, int status)
     callback = (PyObject *)req->data;
 
     if (status != 0) {
-        uv_err_t err = uv_last_error(UV_HANDLE_LOOP(self));
-        py_errorno = PyInt_FromLong(err.code);
+        py_errorno = PyInt_FromLong(status);
     } else {
         py_errorno = Py_None;
         Py_INCREF(Py_None);
@@ -72,14 +70,16 @@ on_tcp_client_connection(uv_connect_t *req, int status)
 static PyObject *
 TCP_func_bind(TCP *self, PyObject *args)
 {
-    int r;
+    int err, flags;
     struct sockaddr_storage ss;
     PyObject *addr;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    if (!PyArg_ParseTuple(args, "O:bind", &addr)) {
+    flags = 0;
+
+    if (!PyArg_ParseTuple(args, "O|i:bind", &addr, &flags)) {
         return NULL;
     }
 
@@ -88,14 +88,9 @@ TCP_func_bind(TCP *self, PyObject *args)
         return NULL;
     }
 
-    if (ss.ss_family == AF_INET) {
-        r = uv_tcp_bind(&self->tcp_h, *(struct sockaddr_in *)&ss);
-    } else {
-        r = uv_tcp_bind6(&self->tcp_h, *(struct sockaddr_in6 *)&ss);
-    }
-
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_bind(&self->tcp_h, (struct sockaddr *)&ss, flags);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -106,10 +101,10 @@ TCP_func_bind(TCP *self, PyObject *args)
 static PyObject *
 TCP_func_listen(TCP *self, PyObject *args)
 {
-    int r, backlog;
+    int err, backlog;
     PyObject *callback, *tmp;
 
-    backlog = 128;
+    backlog = 511;
     tmp = NULL;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -129,9 +124,9 @@ TCP_func_listen(TCP *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_listen((uv_stream_t *)&self->tcp_h, backlog, on_tcp_connection);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_listen((uv_stream_t *)&self->tcp_h, backlog, pyuv__tcp_listen_cb);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -147,7 +142,7 @@ TCP_func_listen(TCP *self, PyObject *args)
 static PyObject *
 TCP_func_accept(TCP *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *client;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -162,9 +157,9 @@ TCP_func_accept(TCP *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_accept((uv_stream_t *)&self->tcp_h, (uv_stream_t *)UV_HANDLE(client));
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_accept((uv_stream_t *)&self->tcp_h, (uv_stream_t *)UV_HANDLE(client));
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -175,7 +170,7 @@ TCP_func_accept(TCP *self, PyObject *args)
 static PyObject *
 TCP_func_connect(TCP *self, PyObject *args)
 {
-    int r;
+    int err;
     struct sockaddr_storage ss;
     uv_connect_t *connect_req = NULL;
     PyObject *addr, *callback;
@@ -207,14 +202,9 @@ TCP_func_connect(TCP *self, PyObject *args)
 
     connect_req->data = callback;
 
-    if (ss.ss_family == AF_INET) {
-        r = uv_tcp_connect(connect_req, &self->tcp_h, *(struct sockaddr_in *)&ss, on_tcp_client_connection);
-    } else {
-        r = uv_tcp_connect6(connect_req, &self->tcp_h, *(struct sockaddr_in6 *)&ss, on_tcp_client_connection);
-    }
-
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_connect(connect_req, &self->tcp_h, (struct sockaddr *)&ss, pyuv__tcp_connect_cb);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         goto error;
     }
 
@@ -233,7 +223,7 @@ error:
 static PyObject *
 TCP_func_getsockname(TCP *self)
 {
-    int r, namelen;
+    int err, namelen;
     struct sockaddr_storage sockname;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -241,20 +231,20 @@ TCP_func_getsockname(TCP *self)
 
     namelen = sizeof(sockname);
 
-    r = uv_tcp_getsockname(&self->tcp_h, (struct sockaddr *)&sockname, &namelen);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_getsockname(&self->tcp_h, (struct sockaddr *)&sockname, &namelen);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
-    return makesockaddr((struct sockaddr *)&sockname, namelen);
+    return makesockaddr((struct sockaddr *)&sockname);
 }
 
 
 static PyObject *
 TCP_func_getpeername(TCP *self)
 {
-    int r, namelen;
+    int err, namelen;
     struct sockaddr_storage peername;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -262,20 +252,20 @@ TCP_func_getpeername(TCP *self)
 
     namelen = sizeof(peername);
 
-    r = uv_tcp_getpeername(&self->tcp_h, (struct sockaddr *)&peername, &namelen);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_getpeername(&self->tcp_h, (struct sockaddr *)&peername, &namelen);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
-    return makesockaddr((struct sockaddr *)&peername, namelen);
+    return makesockaddr((struct sockaddr *)&peername);
 }
 
 
 static PyObject *
 TCP_func_nodelay(TCP *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *enable;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -285,9 +275,9 @@ TCP_func_nodelay(TCP *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_tcp_nodelay(&self->tcp_h, (enable == Py_True) ? 1 : 0);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_nodelay(&self->tcp_h, (enable == Py_True) ? 1 : 0);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -298,7 +288,7 @@ TCP_func_nodelay(TCP *self, PyObject *args)
 static PyObject *
 TCP_func_keepalive(TCP *self, PyObject *args)
 {
-    int r;
+    int err;
     unsigned int delay;
     PyObject *enable;
 
@@ -309,9 +299,9 @@ TCP_func_keepalive(TCP *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_tcp_keepalive(&self->tcp_h, (enable == Py_True) ? 1 : 0, delay);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_keepalive(&self->tcp_h, (enable == Py_True) ? 1 : 0, delay);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -322,7 +312,7 @@ TCP_func_keepalive(TCP *self, PyObject *args)
 static PyObject *
 TCP_func_simultaneous_accepts(TCP *self, PyObject *args)
 {
-    int r;
+    int err;
     PyObject *enable;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -332,9 +322,9 @@ TCP_func_simultaneous_accepts(TCP *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_tcp_simultaneous_accepts(&self->tcp_h, (enable == Py_True) ? 1 : 0);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_simultaneous_accepts(&self->tcp_h, (enable == Py_True) ? 1 : 0);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -345,7 +335,7 @@ TCP_func_simultaneous_accepts(TCP *self, PyObject *args)
 static PyObject *
 TCP_func_open(TCP *self, PyObject *args)
 {
-    int r;
+    int err;
     long fd;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
@@ -355,9 +345,9 @@ TCP_func_open(TCP *self, PyObject *args)
         return NULL;
     }
 
-    r = uv_tcp_open(&self->tcp_h, (uv_os_sock_t)fd);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_TCPError);
+    err = uv_tcp_open(&self->tcp_h, (uv_os_sock_t)fd);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return NULL;
     }
 
@@ -365,10 +355,104 @@ TCP_func_open(TCP *self, PyObject *args)
 }
 
 
+static PyObject *
+TCP_sndbuf_get(TCP *self, void *closure)
+{
+    int err;
+    int sndbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
+
+    sndbuf_value = 0;
+    err = uv_send_buffer_size(UV_HANDLE(self), &sndbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
+        return NULL;
+    }
+    return PyInt_FromLong((long) sndbuf_value);
+}
+
+
+static int
+TCP_sndbuf_set(TCP *self, PyObject *value, void *closure)
+{
+    int err;
+    int sndbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, -1);
+
+    if (!value) {
+        PyErr_SetString(PyExc_TypeError, "cannot delete attribute");
+        return -1;
+    }
+
+    sndbuf_value = (int) PyInt_AsLong(value);
+    if (sndbuf_value == -1 && PyErr_Occurred()) {
+        return -1;
+    }
+
+    err = uv_send_buffer_size(UV_HANDLE(self), &sndbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
+        return -1;
+    }
+    return 0;
+}
+
+
+static PyObject *
+TCP_rcvbuf_get(TCP *self, void *closure)
+{
+    int err;
+    int rcvbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
+
+    rcvbuf_value = 0;
+    err = uv_recv_buffer_size(UV_HANDLE(self), &rcvbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
+        return NULL;
+    }
+    return PyInt_FromLong((long) rcvbuf_value);
+}
+
+
+static int
+TCP_rcvbuf_set(TCP *self, PyObject *value, void *closure)
+{
+    int err;
+    int rcvbuf_value;
+
+    UNUSED_ARG(closure);
+    RAISE_IF_HANDLE_NOT_INITIALIZED(self, -1);
+
+    if (!value) {
+        PyErr_SetString(PyExc_TypeError, "cannot delete attribute");
+        return -1;
+    }
+
+    rcvbuf_value = (int) PyInt_AsLong(value);
+    if (rcvbuf_value == -1 && PyErr_Occurred()) {
+        return -1;
+    }
+
+    err = uv_recv_buffer_size(UV_HANDLE(self), &rcvbuf_value);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
+        return -1;
+    }
+    return 0;
+}
+
+
 static int
 TCP_tp_init(TCP *self, PyObject *args, PyObject *kwargs)
 {
-    int r;
+    int err;
     Loop *loop;
 
     UNUSED_ARG(kwargs);
@@ -379,9 +463,9 @@ TCP_tp_init(TCP *self, PyObject *args, PyObject *kwargs)
         return -1;
     }
 
-    r = uv_tcp_init(loop->uv_loop, &self->tcp_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_TCPError);
+    err = uv_tcp_init(loop->uv_loop, &self->tcp_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_TCPError);
         return -1;
     }
 
@@ -440,6 +524,13 @@ TCP_tp_methods[] = {
 };
 
 
+static PyGetSetDef TCP_tp_getsets[] = {
+    {"send_buffer_size", (getter)TCP_sndbuf_get, (setter)TCP_sndbuf_set, "Send buffer size.", NULL},
+    {"receive_buffer_size", (getter)TCP_rcvbuf_get, (setter)TCP_rcvbuf_set, "Receive buffer size.", NULL},
+    {NULL}
+};
+
+
 static PyTypeObject TCPType = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "pyuv.TCP",                                                    /*tp_name*/
@@ -470,7 +561,7 @@ static PyTypeObject TCPType = {
     0,                                                             /*tp_iternext*/
     TCP_tp_methods,                                                /*tp_methods*/
     0,                                                             /*tp_members*/
-    0,                                                             /*tp_getsets*/
+    TCP_tp_getsets,                                                /*tp_getsets*/
     0,                                                             /*tp_base*/
     0,                                                             /*tp_dict*/
     0,                                                             /*tp_descr_get*/
