@@ -39,16 +39,16 @@ static PyObject *
 Util_func_uptime(PyObject *obj)
 {
     double uptime;
-    uv_err_t err;
+    int err;
     PyObject *exc_data;
 
     UNUSED_ARG(obj);
 
     err = uv_uptime(&uptime);
-    if (err.code == UV_OK) {
+    if (err == 0) {
         return PyFloat_FromDouble(uptime);
     } else {
-        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -62,16 +62,16 @@ static PyObject *
 Util_func_resident_set_memory(PyObject *obj)
 {
     size_t rss;
-    uv_err_t err;
+    int err;
     PyObject *exc_data;
 
     UNUSED_ARG(obj);
 
     err = uv_resident_set_memory(&rss);
-    if (err.code == UV_OK) {
+    if (err == 0) {
         return PyInt_FromSsize_t(rss);
     } else {
-        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -87,45 +87,61 @@ Util_func_interface_addresses(PyObject *obj)
     static char buf[INET6_ADDRSTRLEN+1];
     int i, count;
     uv_interface_address_t* interfaces;
-    uv_err_t err;
+    int err;
     PyObject *result, *item, *exc_data;
 
     UNUSED_ARG(obj);
 
     err = uv_interface_addresses(&interfaces, &count);
-    if (err.code == UV_OK) {
-        result = PyList_New(count);
-        if (!result) {
-            uv_free_interface_addresses(interfaces, count);
-            return NULL;
-        }
-        for (i = 0; i < count; i++) {
-            item = PyStructSequence_New(&InterfaceAddressesResultType);
-            if (!item) {
-                Py_DECREF(result);
-                uv_free_interface_addresses(interfaces, count);
-                return NULL;
-            }
-            PyStructSequence_SET_ITEM(item, 0, Py_BuildValue("s", interfaces[i].name));
-            PyStructSequence_SET_ITEM(item, 1, PyBool_FromLong((long)interfaces[i].is_internal));
-            if (interfaces[i].address.address4.sin_family == AF_INET) {
-                uv_ip4_name(&interfaces[i].address.address4, buf, sizeof(buf));
-            } else if (interfaces[i].address.address4.sin_family == AF_INET6) {
-                uv_ip6_name(&interfaces[i].address.address6, buf, sizeof(buf));
-            }
-            PyStructSequence_SET_ITEM(item, 2, Py_BuildValue("s", buf));
-            PyList_SET_ITEM(result, i, item);
-        }
-        uv_free_interface_addresses(interfaces, count);
-        return result;
-    } else {
-        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+    if (err < 0) {
+        exc_data = Py_BuildValue("(is)", err, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
         }
         return NULL;
     }
+
+    result = PyList_New(count);
+    if (!result) {
+        uv_free_interface_addresses(interfaces, count);
+        return NULL;
+    }
+
+    for (i = 0; i < count; i++) {
+        item = PyStructSequence_New(&InterfaceAddressesResultType);
+        if (!item) {
+            Py_DECREF(result);
+            uv_free_interface_addresses(interfaces, count);
+            return NULL;
+        }
+        PyStructSequence_SET_ITEM(item, 0, Py_BuildValue("s", interfaces[i].name));
+        PyStructSequence_SET_ITEM(item, 1, PyBool_FromLong((long)interfaces[i].is_internal));
+        if (interfaces[i].address.address4.sin_family == AF_INET) {
+            uv_ip4_name(&interfaces[i].address.address4, buf, sizeof(buf));
+        } else if (interfaces[i].address.address4.sin_family == AF_INET6) {
+            uv_ip6_name(&interfaces[i].address.address6, buf, sizeof(buf));
+        }
+        PyStructSequence_SET_ITEM(item, 2, Py_BuildValue("s", buf));
+        if (interfaces[i].netmask.netmask4.sin_family == AF_INET) {
+            uv_ip4_name(&interfaces[i].netmask.netmask4, buf, sizeof(buf));
+        } else if (interfaces[i].netmask.netmask4.sin_family == AF_INET6) {
+            uv_ip6_name(&interfaces[i].netmask.netmask6, buf, sizeof(buf));
+        }
+        PyStructSequence_SET_ITEM(item, 3, Py_BuildValue("s", buf));
+        PyOS_snprintf(buf, sizeof(buf), "%02x:%02x:%02x:%02x:%02x:%02x",
+                                        (unsigned char)interfaces[i].phys_addr[0],
+                                        (unsigned char)interfaces[i].phys_addr[1],
+                                        (unsigned char)interfaces[i].phys_addr[2],
+                                        (unsigned char)interfaces[i].phys_addr[3],
+                                        (unsigned char)interfaces[i].phys_addr[4],
+                                        (unsigned char)interfaces[i].phys_addr[5]);
+        PyStructSequence_SET_ITEM(item, 4, Py_BuildValue("s", buf));
+        PyList_SET_ITEM(result, i, item);
+    }
+
+    uv_free_interface_addresses(interfaces, count);
+    return result;
 }
 
 
@@ -134,13 +150,13 @@ Util_func_cpu_info(PyObject *obj)
 {
     int i, count;
     uv_cpu_info_t* cpus;
-    uv_err_t err;
+    int err;
     PyObject *result, *item, *times, *exc_data;
 
     UNUSED_ARG(obj);
 
     err = uv_cpu_info(&cpus, &count);
-    if (err.code == UV_OK) {
+    if (err == 0) {
         result = PyList_New(count);
         if (!result) {
             uv_free_cpu_info(cpus, count);
@@ -169,7 +185,7 @@ Util_func_cpu_info(PyObject *obj)
         uv_free_cpu_info(cpus, count);
         return result;
     } else {
-        exc_data = Py_BuildValue("(is)", err.code, uv_strerror(err));
+        exc_data = Py_BuildValue("(is)", err, uv_strerror(err));
         if (exc_data != NULL) {
             PyErr_SetObject(PyExc_UVError, exc_data);
             Py_DECREF(exc_data);
@@ -179,151 +195,50 @@ Util_func_cpu_info(PyObject *obj)
 }
 
 
-static void
-getaddrinfo_cb(uv_getaddrinfo_t* req, int status, struct addrinfo* res)
-{
-    PyGILState_STATE gstate = PyGILState_Ensure();
-    struct addrinfo *ptr;
-    Loop *loop;
-    GAIRequest *gai_req;
-    PyObject *addr, *item, *errorno, *dns_result, *result;
-
-    ASSERT(req);
-
-    gai_req = PYUV_CONTAINER_OF(req, GAIRequest, req);
-    loop = REQUEST(gai_req)->loop;
-
-    if (status != 0) {
-        uv_err_t err = uv_last_error(req->loop);
-        errorno = PyInt_FromLong((long)err.code);
-        dns_result = Py_None;
-        Py_INCREF(Py_None);
-        goto callback;
-    }
-
-    dns_result = PyList_New(0);
-    if (!dns_result) {
-        errorno = PyInt_FromLong((long)UV_ENOMEM);
-        dns_result = Py_None;
-        Py_INCREF(Py_None);
-        goto callback;
-    }
-
-    for (ptr = res; ptr; ptr = ptr->ai_next) {
-        addr = makesockaddr(ptr->ai_addr, ptr->ai_addrlen);
-        if (!addr) {
-            PyErr_Clear();
-            break;
-        }
-
-        item = PyStructSequence_New(&AddrinfoResultType);
-        if (!item) {
-            PyErr_Clear();
-            break;
-        }
-        PyStructSequence_SET_ITEM(item, 0, PyInt_FromLong((long)ptr->ai_family));
-        PyStructSequence_SET_ITEM(item, 1, PyInt_FromLong((long)ptr->ai_socktype));
-        PyStructSequence_SET_ITEM(item, 2, PyInt_FromLong((long)ptr->ai_protocol));
-        PyStructSequence_SET_ITEM(item, 3, Py_BuildValue("s", ptr->ai_canonname ? ptr->ai_canonname : ""));
-        PyStructSequence_SET_ITEM(item, 4, addr);
-
-        PyList_Append(dns_result, item);
-        Py_DECREF(item);
-    }
-    errorno = Py_None;
-    Py_INCREF(Py_None);
-
-callback:
-    result = PyObject_CallFunctionObjArgs(gai_req->callback, dns_result, errorno, NULL);
-    if (result == NULL) {
-        handle_uncaught_exception(loop);
-    }
-    Py_XDECREF(result);
-    Py_DECREF(dns_result);
-    Py_DECREF(errorno);
-
-    uv_freeaddrinfo(res);
-    UV_REQUEST(gai_req) = NULL;
-    Py_DECREF(gai_req);
-
-    PyGILState_Release(gstate);
-}
-
 static PyObject *
-Util_func_getaddrinfo(PyObject *obj, PyObject *args, PyObject *kwargs)
+Util_func_getrusage(PyObject *obj)
 {
-    char *host_str;
-    char port_str[6];
-    int port, family, socktype, protocol, flags, r;
-    struct addrinfo hints;
-    Loop *loop;
-    GAIRequest *gai_req;
-    PyObject *callback, *host, *idna;
-
-    static char *kwlist[] = {"loop", "callback", "host", "port", "family", "socktype", "protocol", "flags", NULL};
+    int err;
+    uv_rusage_t ru;
+    PyObject *result;
 
     UNUSED_ARG(obj);
-    gai_req = NULL;
-    idna = NULL;
-    port = socktype = protocol = flags = 0;
-    family = AF_UNSPEC;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!OO|iiiii:getaddrinfo", kwlist, &LoopType, &loop, &callback, &host, &port, &family, &socktype, &protocol, &flags)) {
+    err = uv_getrusage(&ru);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_UVError);
         return NULL;
     }
 
-    if (host == Py_None) {
-        host_str = NULL;
-    } else if (PyUnicode_Check(host)) {
-        idna = PyObject_CallMethod(host, "encode", "s", "idna");
-        if (!idna)
-            return NULL;
-        host_str = PyBytes_AS_STRING(idna);
-    } else if (PyBytes_Check(host)) {
-        host_str = PyBytes_AsString(host);
-    } else {
-        PyErr_SetString(PyExc_TypeError, "getaddrinfo() argument 3 must be string or None");
-        goto error;
+    result = PyStructSequence_New(&RusageResultType);
+    if (!result)
+        return NULL;
+
+#define pyuv__doubletime(TV) ((double)(TV).tv_sec + 1e-6*(TV).tv_usec)
+    PyStructSequence_SET_ITEM(result, 0, PyFloat_FromDouble(pyuv__doubletime(ru.ru_utime)));
+    PyStructSequence_SET_ITEM(result, 1, PyFloat_FromDouble(pyuv__doubletime(ru.ru_stime)));
+    PyStructSequence_SET_ITEM(result, 2, PyLong_FromLong(ru.ru_maxrss));
+    PyStructSequence_SET_ITEM(result, 3, PyLong_FromLong(ru.ru_ixrss));
+    PyStructSequence_SET_ITEM(result, 4, PyLong_FromLong(ru.ru_idrss));
+    PyStructSequence_SET_ITEM(result, 5, PyLong_FromLong(ru.ru_isrss));
+    PyStructSequence_SET_ITEM(result, 6, PyLong_FromLong(ru.ru_minflt));
+    PyStructSequence_SET_ITEM(result, 7, PyLong_FromLong(ru.ru_majflt));
+    PyStructSequence_SET_ITEM(result, 8, PyLong_FromLong(ru.ru_nswap));
+    PyStructSequence_SET_ITEM(result, 9, PyLong_FromLong(ru.ru_inblock));
+    PyStructSequence_SET_ITEM(result, 10, PyLong_FromLong(ru.ru_oublock));
+    PyStructSequence_SET_ITEM(result, 11, PyLong_FromLong(ru.ru_msgsnd));
+    PyStructSequence_SET_ITEM(result, 12, PyLong_FromLong(ru.ru_msgrcv));
+    PyStructSequence_SET_ITEM(result, 13, PyLong_FromLong(ru.ru_nsignals));
+    PyStructSequence_SET_ITEM(result, 14, PyLong_FromLong(ru.ru_nvcsw));
+    PyStructSequence_SET_ITEM(result, 15, PyLong_FromLong(ru.ru_nivcsw));
+#undef pyuv__doubletime
+
+    if (PyErr_Occurred()) {
+        Py_DECREF(result);
+        return NULL;
     }
 
-    if (!PyCallable_Check(callback)) {
-        PyErr_SetString(PyExc_TypeError, "a callable is required");
-        goto error;
-    }
-
-    if (port < 0 || port > 0xffff) {
-        PyErr_SetString(PyExc_ValueError, "port must be between 0 and 65535");
-        goto error;
-    }
-    snprintf(port_str, sizeof(port_str), "%d", port);
-
-    gai_req = (GAIRequest *)PyObject_CallFunctionObjArgs((PyObject *)&GAIRequestType, loop, callback, NULL);
-    if (!gai_req) {
-        PyErr_NoMemory();
-        goto error;
-    }
-
-    memset(&hints, 0, sizeof(hints));
-    hints.ai_family = family;
-    hints.ai_socktype = socktype;
-    hints.ai_protocol = protocol;
-    hints.ai_flags = flags;
-
-    r = uv_getaddrinfo(loop->uv_loop, &gai_req->req, &getaddrinfo_cb, host_str, port_str, &hints);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_UVError);
-        goto error;
-    }
-
-    Py_XDECREF(idna);
-
-    Py_INCREF(gai_req);
-    return (PyObject *)gai_req;
-
-error:
-    Py_XDECREF(idna);
-    Py_XDECREF(gai_req);
-    return NULL;
+    return result;
 }
 
 
@@ -337,7 +252,7 @@ Util_methods[] = {
     { "resident_set_memory", (PyCFunction)Util_func_resident_set_memory, METH_NOARGS, "Gets resident memory size for the current process." },
     { "interface_addresses", (PyCFunction)Util_func_interface_addresses, METH_NOARGS, "Gets network interface addresses." },
     { "cpu_info", (PyCFunction)Util_func_cpu_info, METH_NOARGS, "Gets system CPU information." },
-    { "getaddrinfo", (PyCFunction)Util_func_getaddrinfo, METH_VARARGS|METH_KEYWORDS, "Getaddrinfo" },
+    { "getrusage", (PyCFunction)Util_func_getrusage, METH_NOARGS, "Get information about OS resource utilization for the current process." },
     { NULL }
 };
 
@@ -351,7 +266,7 @@ Util_methods[] = {
 #endif
 
 static int
-drain_poll_fd(uv_os_sock_t fd)
+pyuv__drain_poll_fd(uv_os_sock_t fd)
 {
     static char buffer[1024];
     int r;
@@ -372,7 +287,7 @@ drain_poll_fd(uv_os_sock_t fd)
 
 
 static void
-check_signals(uv_poll_t *handle, int status, int events)
+pyuv__check_signals(uv_poll_t *handle, int status, int events)
 {
     PyGILState_STATE gstate = PyGILState_Ensure();
     SignalChecker *self;
@@ -386,7 +301,7 @@ check_signals(uv_poll_t *handle, int status, int events)
     }
 
     /* Drain the fd */
-    if (drain_poll_fd(self->fd) != 0) {
+    if (pyuv__drain_poll_fd(self->fd) != 0) {
         uv_poll_stop(handle);
     }
 
@@ -405,17 +320,19 @@ check_signals(uv_poll_t *handle, int status, int events)
 static PyObject *
 SignalChecker_func_start(SignalChecker *self)
 {
-    int r;
+    int err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    r = uv_poll_start(&self->poll_h, UV_READABLE, check_signals);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_UVError);
+    err = uv_poll_start(&self->poll_h, UV_READABLE, pyuv__check_signals);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_UVError);
         return NULL;
     }
 
+    PYUV_HANDLE_INCREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -423,17 +340,19 @@ SignalChecker_func_start(SignalChecker *self)
 static PyObject *
 SignalChecker_func_stop(SignalChecker *self)
 {
-    int r;
+    int err;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    r = uv_poll_stop(&self->poll_h);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_UVError);
+    err = uv_poll_stop(&self->poll_h);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_UVError);
         return NULL;
     }
 
+    PYUV_HANDLE_DECREF(self);
+
     Py_RETURN_NONE;
 }
 
@@ -441,7 +360,7 @@ SignalChecker_func_stop(SignalChecker *self)
 static int
 SignalChecker_tp_init(SignalChecker *self, PyObject *args, PyObject *kwargs)
 {
-    int r;
+    int err;
     long fd;
     Loop *loop;
 
@@ -453,9 +372,9 @@ SignalChecker_tp_init(SignalChecker *self, PyObject *args, PyObject *kwargs)
         return -1;
     }
 
-    r = uv_poll_init_socket(loop->uv_loop, &self->poll_h, (uv_os_sock_t)fd);
-    if (r != 0) {
-        RAISE_UV_EXCEPTION(loop->uv_loop, PyExc_UVError);
+    err = uv_poll_init_socket(loop->uv_loop, &self->poll_h, (uv_os_sock_t)fd);
+    if (err < 0) {
+        RAISE_UV_EXCEPTION(err, PyExc_UVError);
         return -1;
     }
 
@@ -572,14 +491,14 @@ init_util(void)
     }
 
     /* initialize PyStructSequence types */
-    if (AddrinfoResultType.tp_name == 0)
-        PyStructSequence_InitType(&AddrinfoResultType, &addrinfo_result_desc);
     if (CPUInfoResultType.tp_name == 0)
         PyStructSequence_InitType(&CPUInfoResultType, &cpu_info_result_desc);
     if (CPUInfoTimesResultType.tp_name == 0)
         PyStructSequence_InitType(&CPUInfoTimesResultType, &cpu_info_times_result_desc);
     if (InterfaceAddressesResultType.tp_name == 0)
         PyStructSequence_InitType(&InterfaceAddressesResultType, &interface_addresses_result_desc);
+    if (RusageResultType.tp_name == 0)
+        PyStructSequence_InitType(&RusageResultType, &rusage_result_desc);
 
     SignalCheckerType.tp_base = &HandleType;
     PyUVModule_AddType(module, "SignalChecker", &SignalCheckerType);
