@@ -6,13 +6,14 @@ sys.path.insert(0, '../')
 import pyuv
 
 
-def on_write2(handle, error):
+def on_write(handle, error):
     global channel, recv_handle
     recv_handle.close()
     channel.close()
 
-def on_channel_read(handle, data, pending, error):
+def on_channel_read(handle, data, error):
     global channel, loop, recv_handle
+    pending = channel.pending_handle_type()
     assert pending in (pyuv.UV_NAMED_PIPE, pyuv.UV_UDP, pyuv.UV_TCP), "wrong handle type"
     if pending == pyuv.UV_NAMED_PIPE:
         recv_handle = pyuv.Pipe(loop)
@@ -21,7 +22,7 @@ def on_channel_read(handle, data, pending, error):
     elif pending == pyuv.UV_UDP:
         recv_handle = pyuv.UDP(loop)
     channel.accept(recv_handle)
-    channel.write2(b".", recv_handle, on_write2)
+    channel.write(b".", on_write, recv_handle)
 
 
 loop = pyuv.Loop.default_loop()
@@ -30,7 +31,7 @@ recv_handle = None
 
 channel = pyuv.Pipe(loop, True)
 channel.open(sys.stdin.fileno())
-channel.start_read2(on_channel_read)
+channel.start_read(on_channel_read)
 
 loop.run()
 
