@@ -1,18 +1,18 @@
 
 import gc
+import unittest
 import weakref
 
-from common import unittest2
+from common import TestCase
 import pyuv
 
 
 class Foo(object): pass
 
-class GCTest(unittest2.TestCase):
+class GCTest(TestCase):
 
     def test_gc(self):
-        loop = pyuv.Loop.default_loop()
-        timer = pyuv.Timer(loop)
+        timer = pyuv.Timer(self.loop)
 
         w_timer = weakref.ref(timer)
         self.assertNotEqual(w_timer(), None)
@@ -29,5 +29,4 @@ class GCTest(unittest2.TestCase):
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
