@@ -1,27 +1,26 @@
 
-from common import unittest2, platform_skip
-from functools import partial
-
-import pyuv
 import random
 import sys
+import unittest
+
+from functools import partial
+
+from common import platform_skip, TestCase
+import pyuv
+
 
 TEST_PORT = random.randint(2000, 3000)
 TEST_PORT2 = random.randint(2000, 3000)
 TEST_ADDR = ("127.0.0.1", TEST_PORT)
 TEST_ADDR2 = ("127.0.0.1", TEST_PORT2)
 
-
 if sys.platform == 'win32':
     TEST_PIPE = '\\\\.\\pipe\\test-pipe'
 else:
     TEST_PIPE = 'test-pipe'
 
 
-class IPCTest(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+class IPCTest(TestCase):
 
     def proc_exit_cb(self, proc, exit_status, term_signal):
         proc.close()
@@ -45,7 +44,8 @@ class IPCTest(unittest2.TestCase):
         self.tcp_server.close()
         self.local_conn_accepted = True
 
-    def on_channel_read(self, handle, data, pending, error):
+    def on_channel_read(self, handle, data, error):
+        pending = self.channel.pending_handle_type()
         if self.tcp_server is None:
             self.assertEqual(pending, pyuv.UV_TCP)
             self.tcp_server = pyuv.TCP(self.loop)
@@ -65,9 +65,11 @@ class IPCTest(unittest2.TestCase):
         self.tcp_server = None
         self.channel = pyuv.Pipe(self.loop, True)
         stdio = [pyuv.StdIO(stream=self.channel, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE|pyuv.UV_WRITABLE_PIPE)]
-        proc = pyuv.Process(self.loop)
-        proc.spawn(file=sys.executable, args=["proc_ipc.py", test_type, str(TEST_PORT)], exit_callback=self.proc_exit_cb, stdio=stdio)
-        self.channel.start_read2(self.on_channel_read)
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_ipc.py", test_type, str(TEST_PORT)],
+                                  exit_callback=self.proc_exit_cb,
+                                  stdio=stdio)
+        self.channel.start_read(self.on_channel_read)
         self.loop.run()
 
     def test_ipc1(self):
@@ -77,15 +79,13 @@ class IPCTest(unittest2.TestCase):
         self._do_test("listen_after_write")
 
 
-class IPCSendRecvTest(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+class IPCSendRecvTest(TestCase):
 
     def proc_exit_cb(self, proc, exit_status, term_signal):
         proc.close()
 
-    def on_channel_read(self, expected_type, handle, data, pending, error):
+    def on_channel_read(self, expected_type, handle, data, error):
+        pending = self.channel.pending_handle_type()
         self.assertEqual(pending, expected_type)
         if pending == pyuv.UV_NAMED_PIPE:
             recv_handle = pyuv.Pipe(self.loop)
@@ -95,30 +95,34 @@ class IPCSendRecvTest(unittest2.TestCase):
             recv_handle = pyuv.UDP(self.loop)
         self.channel.accept(recv_handle)
         self.channel.close()
-        self.send_pipe.close()
+        self.send_handle.close()
         recv_handle.close()
 
     def _do_test(self):
         self.channel = pyuv.Pipe(self.loop, True)
         stdio = [pyuv.StdIO(stream=self.channel, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE|pyuv.UV_WRITABLE_PIPE)]
-        proc = pyuv.Process(self.loop)
-        proc.spawn(file=sys.executable, args=["proc_ipc_echo.py"], exit_callback=self.proc_exit_cb, stdio=stdio)
-        self.channel.write2(b".", self.send_handle)
-        self.channel.start_read2(functools.partial(self.on_channel_read, self.send_handle_type))
+        proc = pyuv.Process.spawn(self.loop,
+                                  args=[sys.executable, "proc_ipc_echo.py"],
+                                  exit_callback=self.proc_exit_cb,
+                                  stdio=stdio)
+        self.channel.write(b".", None, self.send_handle)
+        self.channel.start_read(partial(self.on_channel_read, self.send_handle_type))
         self.loop.run()
 
     @platform_skip(["win32"])
     def test_ipc_send_recv_pipe(self):
         # Handle that will be sent to the process and back
         self.send_handle = pyuv.Pipe(self.loop, True)
-        self.send_handle.bind(TEST_PIPE+'2')
+        self.send_handle.bind(TEST_PIPE + '2')
         self.send_handle_type = pyuv.UV_NAMED_PIPE
+        self._do_test()
 
     def test_ipc_send_recv_tcp(self):
         # Handle that will be sent to the process and back
         self.send_handle = pyuv.TCP(self.loop)
         self.send_handle.bind(TEST_ADDR2)
         self.send_handle_type = pyuv.UV_TCP
+        self._do_test()
 
     @platform_skip(["win32"])
     def test_ipc_send_recv_udp(self):
@@ -126,8 +130,8 @@ class IPCSendRecvTest(unittest2.TestCase):
         self.send_handle = pyuv.UDP(self.loop)
         self.send_handle.bind(TEST_ADDR2)
         self.send_handle_type = pyuv.UV_UDP
+        self._do_test()
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
